/*
 * procdev.h -- functions to deal with /proc/devices and related files.
 *
 * Peter Jones (pjones@redhat.com)
 *
 * Copyright 2007 Red Hat, Inc.
 *
 * This software may be freely redistributed under the terms of the GNU
 * General Public License, version 2.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef NASH_PRIV_PROCDEV_H
#define NASH_PRIV_PROCDEV_H 1

#ifdef _GNU_SOURCE
#define _GNU_SOURCE_DEFINED
#else
#define _GNU_SOURCE 1
#endif

/* FIXME I don't particularly like this being here, but it makes the code
   much cleaner :/ */
typedef enum {
    DEV_TYPE_TREE,
    DEV_TYPE_DM_ERROR,
    DEV_TYPE_DISK,
    DEV_TYPE_DM,
    DEV_TYPE_DM_MPATH,
    DEV_TYPE_PARTITION,
    DEV_TYPE_LVM2_PV,
    DEV_TYPE_LVM2_VG,
    DEV_TYPE_LVM2_LV,
    DEV_TYPE_DM_RAID,
    DEV_TYPE_MD_RAID,
    DEV_TYPE_FS,
    DEV_TYPE_FLOPPY,
    DEV_TYPE_NONE /* end of list marker */
} nash_dev_type;

#define DEV_NAME_EQUALS     0x01
#define DEV_NAME_BEGINS     0x02
#define DEV_MAY_HAVE_PARTS  0x04
#define DEV_MUST_HAVE_PARTS (0x08|DEV_MAY_HAVE_PARTS)
#define DEV_USES_P_CONV     0x10

struct proc_dev_info {
    char name[20];
    int mode;
    int type;
    int flags;
    char prefix[20];
};

struct nash_proc_dev_cache;
extern void nash_proc_dev_cache_destroy(nashContext *nc);

inline int getDevsFromProc(nashContext *nc, int pos, int mode, char **name,
        dev_t *devno, struct proc_dev_info const **info);

#endif /* NASH_PRIV_PROCDEV_H */
/*
 * vim:ts=8:sw=4:sts=4:et
 */
